#!/usr/bin/perl

use File::Copy;
use File::Basename;
use File::Spec;
use File::Path;

$SERVER_ROOT = $ARGV[0];
$BUILD_NUMBER = "2005.311.1230";

sub change_dir {
  my $dir = shift;
  print "Change dir to $dir\n";
  return chdir $dir;
}

sub cmd {
  $c = shift;
  print "$c\n";
  system("$c");
  if( $? ) {
    print "Error: $!, exiting\n";
    exit 1;
  }
}

sub cmd_noerr {
  $c = shift;
  print "$c\n";
  system("$c");
  if( $? ) {
    return 1;
  }
  return 0;
}

sub cp {
  my $c1 = shift; my $c2 = shift;
  print "cp $c1 $c2\n";
  my $res = copy $c1, $c2;
  if( ! $res ) {
    print "Error, $!\n";
  }
  return $res;
}

if (@ARGV != 3) {
	print "Incorrect no: of arguments\n";
	&Usage;
	exit(1);
}

if ( $#ARGV == 0 ) {
  $interactive=1;
}

if ( ! -d $SERVER_ROOT ) {
  print "$SERVER_ROOT directory does not exist\n";
  exit 1;
}

my $CREDS_FILE,$cred_option;

if (! $interactive) {

   $log_file = File::Spec->catfile("$SERVER_ROOT","sync-log");
   
   open(LOG, ">$log_file") or die "Can't create logfile: $!";
   close LOG; 
   $devnul = " >> \"$log_file\" 2>&1 ";

   $CREDS_FILE = File::Spec->catfile("$CUR_DIR", "creds");
   
   if ( "$ARGV[1]" eq "-f" ) {
     if ( ! -f "$ARGV[2]" ) {
        print "Error: $ARGV[2] does not exist\n";
        exit 1;
     }
    cp "$ARGV[2]" , $CREDS_FILE;
   }
   else
   {
    open(CREDS, ">$CREDS_FILE") || die "Could not create : creds, $!";
    print CREDS "Admin Id: $ARGV[1]\n";
    print CREDS "Admin Password: $ARGV[2]\n";
    close CREDS;
   }
 $cred_option=" -f \"$CREDS_FILE\" ";
} else {
 $cred_option="";
}

# Keep trace of presence of a slap- directory
# (which means there's at least one DS installed in 
# the server root.
my $DS_installed = 0;

opendir(SERVERDIR, "$SERVER_ROOT") || die "Could not open directory : $SERVER_ROOT, $!";
@instance_dirs = grep { /^slapd-/ } readdir SERVERDIR;
for (@instance_dirs) {
  my $stop_slapd_dir = File::Spec->catfile("$SERVER_ROOT", $_);
  if( -d $stop_slapd_dir) {
    $DS_installed = 1;
  }
}

if($DS_installed) {
  for (@instance_dirs) {
    my $start_dir = File::Spec->catfile("$SERVER_ROOT","$_");
    if( -d $start_dir) {
      print "cd $start_dir\n";
      change_dir $start_dir;
      my $restart_slapd = File::Spec->catfile(File::Spec->curdir(), "restart-slapd");
      print "Starting instance : $_\n";
      cmd "$restart_slapd";
    }
  }
  
  print "Please wait 20 seconds...\n";
  sleep(20);
}


# Sync the DS.
# Need to do this only if there's at least one
# Directory server configured in SERVER ROOT.
if($DS_installed) {
  $new_dir = File::Spec->catdir("$SERVER_ROOT", "shared", "bin");
  change_dir $new_dir;
  my $sync_product_cds_file = File::Spec->catfile(File::Spec->curdir(), "sync-product-cds");
  my $isie_file = File::Spec->catfile("$SERVER_ROOT","admin-serv", "config", "adm.conf");
  open(ISIE, "$isie_file") || die "$isie_file: $!";
  my ($isie) = grep { /^isie:/i } <ISIE>;
  chop $isie;
  $isie1 = $isie;
  $isie1 =~ s/^[^,]+,(.+)/cn=Sun ONE Directory Server,$1/;

  $sync_product_cds = $sync_product_cds_file . 
    " -r \"$SERVER_ROOT\" -i \"$isie1\" -j ds523.jar -g ds523.jar -v 5.2_Patch_3 -n \"Sun Java(TM) System Directory Server\" -b $BUILD_NUMBER".$cred_option.$devnul;
  print "Updating the Configuration Directory Server for the directory server instances\n";
  $res = cmd_noerr $sync_product_cds;
  if( $res ) {
    print "Warning, sync-product-cds returned with error\n";
    print "Verify user and password are the right ones and that CDS is up and running\n";
    print "You could run the following commands once upgrade is finished:\n";
    print "--------------------------\n";
    print "cd $new_dir\n";
    print "$sync_product_cds_file -r \"$SERVER_ROOT\" -i \"$isie1\" -j ds523.jar -g ds523.jar -v 5.2_Patch_3 -n \"Sun Java(TM) System Directory Server\" -b $BUILD_NUMBER\n";
    print "--------------------------\n";
    $return_code += 1;
  }

  $isie =~ s/^[^,]+,(.+)/cn=Sun Java(TM) System Directory Server,$1/;
  $sync_product_cds = $sync_product_cds_file . 
      " -r \"$SERVER_ROOT\" -i \"$isie\" -j ds523.jar -g ds523.jar -v 5.2_Patch_3 -n \"Sun Java(TM) System Directory Server\" -b $BUILD_NUMBER".$cred_option.$devnul;
  system($sync_product_cds);
  $isie =~ s/^[^,]+,(.+)/cn=Sun ONE System Directory Server,$1/;
  $sync_product_cds = $sync_product_cds_file . 
      " -r \"$SERVER_ROOT\" -i \"$isie\" -j ds523.jar -g ds523.jar -v 5.2_Patch_3 -n \"Sun Java(TM) System Directory Server\" -b $BUILD_NUMBER".$cred_option.$devnul;
  system($sync_product_cds);

}

# And start the admin server

print "Starting the admin server...\n";
change_dir $SERVER_ROOT;
my $start_admin = File::Spec->catfile(File::Spec->curdir(), "start-admin");
$return_code += cmd_noerr "$start_admin";

print "Stoping the admin server...\n";
change_dir $SERVER_ROOT;
my $stop_admin = File::Spec->catfile(File::Spec->curdir(), "stop-admin");
$return_code += cmd_noerr "$stop_admin";

if (! $interactive) {
 unlink "$CREDS_FILE";
}

print "\n\nDone\n\n";

if ($return_code == 0) {
  unlink "$log_file";
}

# We need first to stop all instances of the server
opendir(SERVERDIR, "$SERVER_ROOT") || die "Could not open directory : $SERVER_ROOT, $!";
@instance_dirs = grep { /^slapd-/ } readdir SERVERDIR;
for (@instance_dirs) {
  my $stop_slapd_dir = File::Spec->catfile("$SERVER_ROOT", $_);
  if( -d $stop_slapd_dir) {
    $DS_installed = 1;
    change_dir $stop_slapd_dir || die "Could not change dir to $stop_slapd_dir";
    print "Stopping instance : $_\n";
    my $stop_slapd = File::Spec->catfile(File::Spec->curdir(), "stop-slapd");
    $res = cmd_noerr "$stop_slapd";
    if ( $res )	 {
	$return_code += 1;
    }
  }
}